﻿import { Injectable } from '@angular/core'

import { AcquisitionGroupModel, AcquisitionStepModel, CharacteristicType } from '../model/acquisitionGroup.model'
import { IUiElement } from '../model/ui.model'
import { IteratorService } from './iterator.service'
//import { AcquisitionValidationService } from './acq.validation.service'
import { Constants } from '../Constants'
import { HtmlCreationService } from './htmlCreation.service'

@Injectable()
export class AcquisitionTableUpdateService {

    private defaultLineBar = "<div class = 'line-color-notdone' ></div>";

    /**
     * Constructor.
     * @param iteratorService
     * @param validationService
     * @param htmlCreationService
     */
    constructor(private iteratorService: IteratorService, private htmlCreationService: HtmlCreationService) { }

    /**
     * Resets all acquisition groups.
     * @param acquisitionGroups
     */
    resetGroups(acquisitionGroups: Array<AcquisitionGroupModel>) {
        for (let i = 1; i <= acquisitionGroups.length; i++) {
            $("#hgrid1").igGridUpdating("updateRow", i, { "AcqStatus": Constants.NO, "Value": null, "Chart": this.defaultLineBar });
        }
    }

    setParentLineBar($event : any, ui: IUiElement) {

        if ($event.parent.Type == CharacteristicType.VAR) {
            this.setParentDataVISVAR($event, ui);
        }
        else if ($event.parent.Type == CharacteristicType.ATT) {
            this.setATTParentRow($event, ui);
        }
        else if ($event.parent.Type == CharacteristicType.VIS) {
            this.setParentDataVISVAR($event, ui);
        }
    }
    /**
     * Sets line bar for given acquisition group.
     * @param parent
     * @param ui
     */
    setLineBar(parent: AcquisitionGroupModel, child: AcquisitionStepModel, ui: IUiElement) {
        if (parent.Type == CharacteristicType.VAR) {
            this.setLineBarVar(parent, child, ui.value, ui.rowID);
            this.updateVAR(ui, ui.value, child.ChartValue);
            //this.setParentDataVISVAR(parent, ui);
        }
        else if (parent.Type == CharacteristicType.ATT) {
            this.setLineBarATT(parent, child, child.Measurement, ui.rowID);
            this.upadteATT(ui, child.IO, child.Measurement, child.ChartValue);
            //this.setATTParentRow(parent, ui);
        }
        else if (parent.Type == CharacteristicType.VIS) {
            this.setLineBarVis(parent, child, ui.rowID);
            this.updateVIS(ui, child.Measurement, child.ChartValue);
            //this.setParentDataVISVAR(parent, ui);
        }
    }

    /**
     * Sets parent data for VIS and VAR groups.
     * @param parent
     * @param ui
     */
    private setParentDataVISVAR($event, ui: IUiElement) {

        var total: number = 0; var count: number = 0;
        var successCnt: number = 0; var failureCnt: number = 0;
        var notDone: number = 0;
        var numOfStep: number = $event.parent.AcquisitionSteps.length;
        var step =  $event.child.rowID;
        for (var i = 0; i < numOfStep; i++) {
            let acqVal = $event.parent.AcquisitionSteps[i].Measurement;
            if (acqVal != null) {
                total += acqVal;
                if (this.checkForOutOfLimit($event.parent, i, acqVal))
                    successCnt++;
                else
                    failureCnt++;
                count++
            }

        }

        var avg = count == 0 ? "" : Math.floor(successCnt / count * 100) + "%";
        notDone = numOfStep - successCnt - failureCnt;
        successCnt = Math.abs(successCnt / numOfStep * 95);
        failureCnt = Math.abs(failureCnt / numOfStep * 95);
        notDone = Math.abs(notDone / numOfStep * 95);
        var html: string = this.getParentRowHTML(successCnt, failureCnt, notDone);
        $event.parent.Value = total;
        $event.parent.Chart = html;
        if (count == $event.parent.AcquisitionSteps.length)
            $event.parent.AcqStatus = Constants.YES;
        else
            $event.parent.AcqStatus = Constants.NO;

        ui.updateParentRow({ "AcqStatus": $event.parent.AcqStatus, "Value": avg, "Chart": html });
    }

    /**
     * Gets HTML string for parent row.
     * @param successPercent
     * @param failurePercent
     * @param notDone
     */
    private getParentRowHTML(successPercent: number, failurePercent: number, notDone: number): string {
        var strHtml: string = "<span style='display: -webkit-box'><div class = 'line-color-success' style='width:" + successPercent + "%'></div><div class = 'line-color-danger'  style='width:" + failurePercent + "%'></div><div class = 'line-color-notdone'  style='width:" + notDone + "%'></div></span>";
        return strHtml;
    }

    /**
     * Sets parent row for ATT type.
     * @param parent
     * @param ui
     */
    private setATTParentRow($event, ui: IUiElement) {
        var child: AcquisitionStepModel = $event.child;
        var html = child.ChartValue;
        var count: number = 0;
        var numOfStep: number = $event.parent.AcquisitionSteps.length;

        if (null != $event.parent.AcquisitionSteps[0].Measurement)
            $event.parent.AcqStatus = Constants.YES;
        else
            $event.parent.AcqStatus = Constants.NO;

        var avg: string = Math.floor(child.IO / child.AmountOfParts * 100) + "%";
        ui.updateParentRow({ "AcqStatus": $event.parent.AcqStatus, "Value": avg, "Chart": html });
    }

    /**
     * Update ATT row.
     * @param ui
     * @param IO
     * @param NIO
     * @param ChartValue
     */
    private upadteATT(ui: IUiElement, IO: number, NIO: number, ChartValue: string) {
        ui.updateRow({
            "IO": IO,
            "NIO": NIO,
            "ChartValue": ChartValue
        });
    }

    /**
     * Updates VIS row.
     * @param ui
     * @param defectCount
     * @param ChartValue
     */
    private updateVIS(ui: IUiElement, defectCount: any, ChartValue: string) {
        ui.updateRow({
            "DefectCount": defectCount,
            "ChartValue": ChartValue
        });
    }

    /**
     * Updates VAR row.
     * @param ui
     * @param acqValue
     * @param chartValue
     */
    private updateVAR(ui: IUiElement, acqValue: number, chartValue: string) {
        ui.updateRow({
            "ACQValue": acqValue,
            "ChartValue": chartValue
        });
    }

    /**
     * Sets line bar for ATT.
     * @param agtObj
     * @param value
     * @param currentRowID
     */
    private setLineBarATT(agtObj: AcquisitionGroupModel, child: AcquisitionStepModel, value: number, currentRowID: number) {
        var child: AcquisitionStepModel = child;
        var numberOfParts = Number(child.AmountOfParts);
        if (value == null) {
            child.IO = null;
            child.Measurement = null;
            child.ChartValue = this.defaultLineBar;
        } else {
            if (numberOfParts < 0 || numberOfParts < value) {
                var successPercent: number = Math.abs(numberOfParts / numberOfParts * 95);
                var HTML: string = this.createAttHtmlString(successPercent, 0);
                child.IO = 0;
                child.Measurement = numberOfParts;
                child.ChartValue = HTML;
            } else if (0 <= numberOfParts && value <= numberOfParts) {
                var value: number = numberOfParts - Number(child.Measurement);
                var failurePercent: number = Math.abs(child.Measurement / numberOfParts * 95);
                var HTML: string = this.createAttHtmlString(95 - failurePercent, failurePercent);
                child.IO = value;
                child.ChartValue = HTML;
            }
        }
    }

    /**
     * Sets line bar for VIS.
     * @param parent
     * @param currentRowID
     */
    private setLineBarVis(parent: AcquisitionGroupModel, child: AcquisitionStepModel, currentRowID: number) {
        var acqStep = child;
        if (acqStep.Measurement == null) {
            acqStep.ChartValue = this.defaultLineBar;
        } else {
            acqStep.Measurement = 0;
            acqStep.ChartValue = "<div class = 'line-color-success' ></div> ";
        }
    }

    /**
     * Sets line bar for VAR.
     * @param agtObj
     * @param value
     * @param currentRowID
     */
    private setLineBarVar(agtObj: AcquisitionGroupModel, child: AcquisitionStepModel, value: number, currentRowID: number) {
        var currentchild: AcquisitionStepModel = child;
        var ul: number = currentchild.UpperTolerance;
        var ll: number = currentchild.LowerTolerance;
        let nominalVal: number = currentchild.NominalValue;
        let margin: number;
        if (ll != null) {
            //set margin for left vericle line to 5% 
            margin = 5;
        }
        else {
            margin = 0;
        }
        var lineHtml: string = "<span style='display: -webkit-box'>" + this.htmlCreationService.createVARHtml(ul, ll, value, nominalVal, margin, Constants.MIN_LINE_WIDTH,3); + "</span>";
        currentchild.ChartValue = lineHtml;
    }

    /**
     * Checks if given value is outside limits.
     * @param parent
     * @param index
     * @param val
     */
    private checkForOutOfLimit(parent: AcquisitionGroupModel, index: number, val: number): boolean {
        if (val <= parent.AcquisitionSteps[index].UpperTolerance && val >= parent.AcquisitionSteps[index].LowerTolerance)
            return true;
        return false;
    }

    /**
     * Creates HTML for ATT.
     * @param successPercent
     * @param failurePercent
     */
    private createAttHtmlString(successPercent: number, failurePercent: number): string {
        var strHtml: string = "<span style='display: -webkit-box'><div class = 'line-color-success' style='width:" + successPercent + "%'></div><div class = 'line-color-danger'  style='width:" + failurePercent + "%'></div></span>";
        return strHtml;
    }
}